/*******************************************************************************
 *
 * Copyright (c) 2016-2026 HYCON Technology, Inc.
 * All rights reserved.
 * HYCON Technology <www.hycontek.com>
 *
 * HYCON reserves the right to amend this code without notice at any time.
 * HYCON assumes no responsibility for any errors appeared in the code,
 * and HYCON disclaims any express or implied warranty, relating to sale
 * and/or use of this code including liability or warranties relating
 * to fitness for a particular purpose, or infringement of any patent,
 * copyright or other intellectual property right.
 *
 * -----------------------------------------------------------------------------
 * File Name    : user_DEF.h
 * IDE tooling  : AndeSight C/C++ IDE, version: 2.0.1 Build ID : 201303201736
 * Device Ver.  : V0.9 crt0.o for HY16F19xx & HY16F18xx MCU.
 * Library Ver. : 1.1
 * MCU Device   : HY16F198B-L100 @3.0V
 * Description  : This file implements the customer's function.
 * Created Date : 2016/05/28
 * Created by   :
 *
 ******************************************************************************/

#ifndef _USER_DEF_H_
#define _USER_DEF_H_

/*----------------------------------------------------------------------------*/
/* Header file include                                                        */
/*----------------------------------------------------------------------------*/
#include "DrvREG32.h"
#include "SpecialMacro.h"
//#include "stdint.h"

/*----------------------------------------------------------------------------*/
/* Structure definition                                                       */
/*----------------------------------------------------------------------------*/
volatile typedef union _USERFLAG0
{
    char _byte;
    struct
    {
        unsigned b_InitGo_Flag:1;
        unsigned b_EntSLP_Flag:1;
        unsigned b_KInPro_Flag:1;
        unsigned b_KeyOn_Flag:1;
        unsigned b_TB8mS_Flag:1;
        unsigned b_PowOn_Flag:1;
        unsigned b_CalGo_Flag:1;
        unsigned b_bit7:1;
    };
} USERFLAG0;

volatile typedef union _USERFLAG1
{
    char _byte;
    struct
    {
        unsigned b_ShowAD_Flag:1;
        unsigned b_ADC_In_Flag:1;
        unsigned b_DotKey_Flag:1;
        unsigned b_TareEI_Flag:1;
        unsigned b_CalEI_Flag:1;
        unsigned b_bit5:1;
        unsigned b_bit6:1;
        unsigned b_bit7:1;
    };
} USERFLAG1;

/*----------------------------------------------------------------------------*/
/* Global variable definition                                                 */
/*----------------------------------------------------------------------------*/
USERFLAG0 UserFlag0;
USERFLAG1 UserFlag1;
extern unsigned char DisplayBuffer[18];
extern unsigned int  Sleep_Time_Cnt;
extern unsigned int  ADC_RawData;

/*----------------------------------------------------------------------------*/
/* Constant value definition                                                  */
/*----------------------------------------------------------------------------*/
#define BIT0                    0x01                    //
#define BIT1                    0x02
#define BIT2                    0x04
#define BIT3                    0x08
#define BIT4                    0x10
#define BIT5                    0x20
#define BIT6                    0x40
#define BIT7                    0x80

#define INIT_GO_TIME            ( 700/8)                //Power initial delay time count= ( 700mS)/(8mS time base).
#define PWM_4KHZ_TIME           (4000000u/4000u)        //Time count= HS_CK/4KHz.
#define PWM_2KHZ_TIME           (4000000u/2000u)        //Time count= HS_CK/2KHz.
#define SLEEP_TIME              (180000u/32u)           //Time count= (180Sec)/(32mS time base).
//#define SLEEP_TIME              (10000u/32u)            //Time count= (10Sec)/(32mS time base).
#define NO_KEY_RESET_TIME       (1000u/32u)             //Time count= (1000mS)/(32mS time base).
#define BEEP_TONE_TIME          (80u/8u)                //Time count= (beep time)/(8mS time base).
#define BEEP_TONE_ENABLE        TRUE                    //Beep tone enable/disable.
//#define WDT_RESET_ENABLE        FALSE                   //WDT reset function  enable/disable.
#define BURN_VERIFY_TIMES       3
#define CAL_0KG_BUF_ADDR        0xFE00                  //Data flash address for 0Kg calibrated valuation.
#define CAL_2KG_BUF_ADDR        0xFE04                  //Data flash address for 2Kg calibrated valuation.

#define S1_KEY_NO               0                       //S1  key no.
#define S2_KEY_NO               4                       //S2  key no.
#define S3_KEY_NO               8                       //S3  key no.
#define S4_KEY_NO               12                      //S4  key no.
#define S5_KEY_NO               1                       //S5  key no.
#define S6_KEY_NO               5                       //S6  key no.
#define S7_KEY_NO               9                       //S7  key no.
#define S8_KEY_NO               13                      //S8  key no.
#define S9_KEY_NO               2                       //S9  key no.
#define S10_KEY_NO              6                       //S10 key no.
#define S11_KEY_NO              10                      //S11 key no.
#define S12_KEY_NO              14                      //S12 key no.
#define S13_KEY_NO              3                       //S13 key no.
#define S14_KEY_NO              7                       //S14 key no.
#define S15_KEY_NO              11                      //S15 key no.
#define S16_KEY_NO              15                      //S16 key no.
#define NULL_KEY_NO             0xFF                    //null key no.
#define KEY_DB_TIME             (65u/8u)                //Debounce time count= TIME/(8mS time base).

/*----------------------------------------------------------------------------*/
/* Function prototype declaration                                             */
/*----------------------------------------------------------------------------*/

extern void Delay(unsigned int num);
extern void User_Sample_Init(void);
extern void PowerOn_Init_Go(void);
extern void Enter_SLEEP_Mode(void);
extern void Release_SLEEP_Mode(void);
extern void ADC_RawData_Avg_Process(void);
//extern void Check_Cal_Mode_SW(void);
extern void Key_Scan_Process(void);
extern void Beep_Tone_Process(void);
extern void Key_In_Process(void);
extern void Get_Weight_Nums(void);
extern void LCD_Display_Process(void);

#endif  //_USER_DEF_H_
