/*******************************************************************************
 *
 * Copyright (c) 2016-2026 HYCON Technology, Inc.
 * All rights reserved.
 * HYCON Technology <www.hycontek.com>
 *
 * HYCON reserves the right to amend this code without notice at any time.
 * HYCON assumes no responsibility for any errors appeared in the code,
 * and HYCON disclaims any express or implied warranty, relating to sale
 * and/or use of this code including liability or warranties relating
 * to fitness for a particular purpose, or infringement of any patent,
 * copyright or other intellectual property right.
 *
 * -----------------------------------------------------------------------------
 * File Name    : user_sample.c
 * IDE tooling  : AndeSight C/C++ IDE, version: 2.0.1 Build ID : 201303201736
 * Device Ver.  : V0.9 crt0.o for HY16F19xx & HY16F18xx MCU.
 * Library Ver. : 1.1
 * MCU Device   : HY16F198B-L100 @3.0V
 * Description  : This file implements the customer's function.
 * Created Date : 2016/05/28
 * Created by   :
 *
 ******************************************************************************/

/*----------------------------------------------------------------------------*/
/* Header file include                                                        */
/*----------------------------------------------------------------------------*/
//#include "stdint.h"
#include "HY16F198.h"
#include "System.h"
#include "DrvADC.h"
#include "DrvClock.h"
#include "DrvPMU.h"
#include "DrvGPIO.h"
#include "DrvLCD.h"
#include "DrvREG32.h"
#include "DrvRTC.h"
#include "DrvTimer.h"
#include "DrvUART.h"
#include "DrvFlash.h"
#include "ModuleID.h"
#include "SpecialMacro.h"
#include "Sysinfra.h"
#include "user_DEF.h"
#include "user_LCD.h"

/*----------------------------------------------------------------------------*/
/* Structure definition                                                       */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* Global variable definition                                                 */
/*----------------------------------------------------------------------------*/
unsigned int  Sleep_Time_Cnt;
unsigned char DisplayBuffer[18];
unsigned char Beep_Tone_Cnt;
unsigned char Key_Code_Buf;
unsigned char Key_Code_No;
unsigned char Key_DB_Cnt;
unsigned int  ADCbuffer[16];
unsigned int  CAL_0Kg_Buf;
unsigned int  CAL_2Kg_Buf;
unsigned int  GradientBuf;
unsigned int  DecorticateBuf;
unsigned int  Total_Price;
unsigned int  Unit_Price;
unsigned int  Weight_Nums;
unsigned int  ADC_RawData;
unsigned int  RawData_Temp;
unsigned int  ADC_Avg_RawData;
unsigned int  ADC_Zero;
unsigned int  ADC_nums;
unsigned int  FIR_SumBuf;
unsigned char FIR_Buf_Idx;
//unsigned char Throw_Cnt;
unsigned int  NoKey_Reset_Cnt;
unsigned int  NoKeyBuf[4];
unsigned char NoBuf_Idx;
unsigned int  CAL_Step_Idx;

/*----------------------------------------------------------------------------*/
/* Constant value definition                                                  */
/*----------------------------------------------------------------------------*/
const unsigned char code_seg[] =
{
    seg_a+seg_b+seg_c+seg_d+seg_e+seg_f,        //00, char "0"
    seg_b+seg_c,                                //01, char "1"
    seg_a+seg_b+seg_d+seg_e+seg_g,              //02, char "2"
    seg_a+seg_b+seg_c+seg_d+seg_g,              //03, char "3"
    seg_b+seg_c+seg_f+seg_g,                    //04, char "4"
    seg_a+seg_c+seg_d+seg_f+seg_g,              //05, char "5"
    seg_a+seg_c+seg_d+seg_e+seg_f+seg_g,        //06, char "6"
    seg_a+seg_b+seg_c,                          //07, char "7"
    seg_a+seg_b+seg_c+seg_d+seg_e+seg_f+seg_g,  //08, char "8"
    seg_a+seg_b+seg_c+seg_d+seg_f+seg_g,        //09, char "9"
    seg_a+seg_b+seg_c+seg_e+seg_f+seg_g,        //0A, char "A"
    seg_c+seg_d+seg_e+seg_f+seg_g,              //0B, char "b"
    seg_a+seg_d+seg_e+seg_f,                    //0C, char "C"
    seg_b+seg_c+seg_d+seg_e+seg_g,              //0D, char "d"
    seg_a+seg_d+seg_e+seg_f+seg_g,              //0E, char "E"
    seg_a+seg_e+seg_f+seg_g                     //0F, char "F"
};

/*----------------------------------------------------------------------------*/
/* Function prototype declaration                                             */
/*----------------------------------------------------------------------------*/

//------MISC Function declaration---------------
void User_Sample_Init(void);
void Delay(unsigned int num);
void Beep_Tone_Start(void);
void Beep_Tone_Stop(void);
void Key_Scan_Process(void);
void Key_In_Process(void);
int DrvFlash_WriteWord(unsigned int data_addr, unsigned int data_buf);
void Beep_Tone_Process(void);
void PowerOn_Init_Go(void);
void Enter_SLEEP_Mode(void);
void Release_SLEEP_Mode(void);
void Get_Weight_Nums(void);
unsigned int Check_Origin(void);

//------ADC Function declaration----------------
void ADC_Init(void);
void ADC_RawData_Avg_Process(void);

//------GPIO Function declaration---------------
void GPIO_Init(void);

//------TIMER Function declaration--------------
void TIMER_Init(void);
void WDT_Init(void);

//------LCD Function declaration----------------
void LCD_Init(void);
void LCD_Display_Process(void);
void LCD_RAM_Clear(unsigned char SegBufSize);
void LCD_Data_Display(unsigned char lcd_no, unsigned int LcdBuffer);
void LCD2_Display_HYcon(void);
void LCD3_Display_SCALE(void);
void LCD3_Display_ADC(unsigned int adc_data);
void LCD3_Display_PASS(void);
void LCD3_Display_CAL(unsigned char kg_num);
void RAM2LCD( unsigned char *Buffer_Adr );

/*----------------------------------------------------------------------------*/
/* Function Name: User_Sample_Init                                            */
/* Description  : Sample code Initialization Subroutines                      */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void User_Sample_Init(void)
{
    unsigned int InitGo_cnt;

    SYS_DisableGIE();                           //Disable Global Interrupt.

//------Sys. clock setting----------------------
    DrvCLOCK_SelectIHOSC( TRIM_HAO4MHZ );       //Select HAO= 4MHz.
    DrvCLOCK_EnableHighOSC(E_INTERNAL, 60);     //Enable HAO.
    DrvCLOCK_SelectMCUClock(0, 0);              //Select MCUCKS= HS_CK/1.

//------MISC. system initial--------------------
    GPIO_Init();
    LCD_Init();
    ADC_Init();
    TIMER_Init();
    UserFlag0._byte = 0;
    SYS_EnableGIE( 4, 0x06 );                   //Enable HW1 & HW2 Int. only.

//------Power on initial going------------------
//Beep tone on & LCD display HYcon SCALE about 1Sec.
//
    Beep_Tone_Start();
    LCD2_Display_HYcon();
    LCD3_Display_SCALE();
    RAM2LCD( DisplayBuffer );                   //Update LCD display RAM.
    for ( InitGo_cnt= 0; InitGo_cnt < INIT_GO_TIME; )
    {
        if ( UserFlag0.b_TB8mS_Flag == TRUE )
        {
            UserFlag0.b_TB8mS_Flag = FALSE;
            InitGo_cnt++;
        }
    }
//  Beep_Tone_Stop();
    LCD_RAM_Clear( SEG_BUF_SIZE );
    UserFlag0.b_InitGo_Flag = TRUE;
}

/*----------------------------------------------------------------------------*/
/* Function Name: PowerOn_Init_Go                                             */
/* Description  : Initial all user tasks (likes LCD, ADC for scale etc),      */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void PowerOn_Init_Go(void)
{
    unsigned int i, j;

    SYS_DisableGIE();                           //Disable Global Interrupt.

//------MISC data initial-----------------------
    GPIO_Init();
    ADC_Init();
    LCD_Init();
    TIMER_Init();

    UserFlag1._byte = 0;
    Total_Price = 0;
    DecorticateBuf = 0;
    NoKey_Reset_Cnt = 0;
    NoBuf_Idx = 2;
    NoKeyBuf[0] = 0;
    NoKeyBuf[1] = 0;
    NoKeyBuf[2] = 0;
    NoKeyBuf[3] = 0;

    CAL_0Kg_Buf = ReadWord( CAL_0KG_BUF_ADDR );
    CAL_2Kg_Buf = ReadWord( CAL_2KG_BUF_ADDR );

//------Enter Calibration mode ?----------------
    for ( j= 1000, i=0; i < 500; i++ )
    {
        Delay( 10 );                                            //delay 18uS @4MHz.
        if ( DrvGPIO_GetPortBits( E_PT2 ) & 0x04 )
            j--;
        else
            j++;
    }
    if ( j > 1400 ) { UserFlag1.b_CalEI_Flag = TRUE;}
    DrvGPIO_PT2_DisableINPUT( 0x04 );                           //Disable PT2.2 IE.
    DrvGPIO_PT2_DisablePullHigh( 0x04 );                        //Disable PT2.2 pull high.

    if( (CAL_0Kg_Buf == 0) || (CAL_2Kg_Buf == 0) || ( CAL_0Kg_Buf > CAL_2Kg_Buf) || ( CAL_0Kg_Buf == CAL_2Kg_Buf) )
    {
        UserFlag1.b_CalEI_Flag = TRUE;          //enable Calibration mode.
    }

//------System initial go !---------------------
    SYS_EnableGIE( 4, 0x06 );                   //Enable HW1 & HW2 Int. only.

    if ( UserFlag0.b_PowOn_Flag == FALSE )
    {
        //enable Comb Filter.
        DrvADC_CombFilter( 0 );                 //reset !
        DrvADC_CombFilter( 1 );                 //enable !

        for(i=0; i<16; i++)
        {
            ADCbuffer[i] = 0;
        }

        FIR_SumBuf = 0;
        FIR_Buf_Idx = 0;
        Unit_Price = 0;
        Weight_Nums = 0;

        //initial FIR buffer.
        while( UserFlag1.b_ADC_In_Flag == FALSE );
        UserFlag1.b_ADC_In_Flag = FALSE;
        for( i=0; i<8; i++)
        {
            ADCbuffer[i]= ADC_RawData;
            FIR_SumBuf += ADCbuffer[i];
            ADCbuffer[i]= ADC_RawData;
        }
        ADC_Avg_RawData = ADC_RawData;
        ADC_Zero = ADC_Avg_RawData;
    }
    //fast update ADC data, if ADC Raw data is changed more than 400 counts.
    else if ( ( UserFlag0.b_PowOn_Flag == TRUE ) && ( UserFlag0.b_CalGo_Flag == FALSE ) )
    {
        for ( i=0, j=0; i<4; i++ )
        {
            while( UserFlag1.b_ADC_In_Flag == FALSE );
            UserFlag1.b_ADC_In_Flag = FALSE;
            if ( ( ADC_RawData > RawData_Temp+30 ) || ( ADC_RawData < RawData_Temp+30 ) )
                j++;
            else { j = 0; }
        }

        if ( j == 4 )
        {
            FIR_SumBuf = 0;
            FIR_Buf_Idx = 0;
            for( i=0; i<8; i++)
            {
                ADCbuffer[i]= ADC_RawData;
                FIR_SumBuf += ADCbuffer[i];
                ADCbuffer[i]= ADC_RawData;
            }
            ADC_Avg_RawData = ADC_RawData;
            ADC_nums = ADC_RawData;
        }
        Beep_Tone_Start();
    }

    UserFlag0.b_PowOn_Flag = TRUE;
    UserFlag0.b_CalGo_Flag = FALSE;
    UserFlag0.b_InitGo_Flag = FALSE;
    Sleep_Time_Cnt = SLEEP_TIME;
    LCD_Data_Display( LCD_PANEL1, Unit_Price );
    LCD_Data_Display( LCD_PANEL2, Total_Price );
    LCD_Data_Display( LCD_PANEL3, Weight_Nums );
    WDT_Init();
}

/*----------------------------------------------------------------------------*/
/* Function Name: Get_Weight_Nums                                             */
/* Description  : Moving average (FIR filter) for ADC raw datas.              */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       : average times = 16 times and ignore first three ADC datas.  */
/*----------------------------------------------------------------------------*/
void Get_Weight_Nums(void)
{
//------Renew Origin----------------------------
    if( ADC_Avg_RawData < ADC_Zero )
    {
        ADC_Zero = ADC_Avg_RawData;
    }
    else
    {
        if( ADC_Avg_RawData < (ADC_Zero + 3) )
        {
            ADC_Avg_RawData = ADC_Zero;
        }
    }

//------Get Gradient (means delta-ADC)----------
    if(ADC_nums > ADC_Avg_RawData)
    {
        if( ADC_nums > (ADC_Avg_RawData + 3) )
        {
            ADC_nums = ADC_Avg_RawData;
        }
    }
    else
    {
        if( ADC_Avg_RawData > (ADC_nums + 3) )
        {
            ADC_nums = ADC_Avg_RawData;
        }
    }

    if ( ADC_nums < ADC_Zero ) { ADC_nums = ADC_Zero; }
    GradientBuf = ADC_nums - ADC_Zero;

//------Get current Weight----------------------
    if ( UserFlag1.b_CalEI_Flag == FALSE )
    {
        Weight_Nums = 0;
        if( (CAL_0Kg_Buf != 0) && (CAL_2Kg_Buf != 0) && (CAL_2Kg_Buf > CAL_0Kg_Buf+500) && (CAL_2Kg_Buf != CAL_0Kg_Buf) )
            Weight_Nums = (GradientBuf*2000) / (CAL_2Kg_Buf - CAL_0Kg_Buf);                 //p`qAg
    }
    else
    {
        Weight_Nums = GradientBuf;                          //uADCƭ
    }

    if( Weight_Nums < DecorticateBuf )
    {
        Weight_Nums = 0;
    }
    else
    {
        Weight_Nums = Weight_Nums - DecorticateBuf;
    }

    //Reset Sleep Counter?
    if ( ( RawData_Temp > ADC_Avg_RawData + 20 ) || ( RawData_Temp + 20 < ADC_Avg_RawData ) )
        Sleep_Time_Cnt = SLEEP_TIME;
    RawData_Temp = ADC_Avg_RawData;
}

/*----------------------------------------------------------------------------*/
/* Function Name: Check_Origin                                                */
/* Description  : Get the curent origin from the stable ADC data.             */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
unsigned int Check_Origin(void)
{
    unsigned int checknum = 0, checkbuf = 0;
    while( checknum < 100 )
    {
        ADC_RawData_Avg_Process();
        if( -5 < (int)checkbuf-(int)ADC_Avg_RawData || (int)checkbuf-(int)ADC_Avg_RawData < 5)
        { checknum += 1; }
        else
            checknum = 0;
        checkbuf = ADC_Avg_RawData;
    }
    return checkbuf;
}

/*----------------------------------------------------------------------------*/
/* Function Name: Key_Scan_Process                                            */
/* Description  : 4x4 key matrix scanning.                                    */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*                                                                            */
/* 4x4 KEY Matrix                                                             */
/*                ------------------------------------------< PT1.7           */
/*                |         --------------------------------< PT1.6           */
/*                |         |         ----------------------< PT1.5           */
/*                |         |         |         ------------< PT1.4           */
/*                |         |         |         |                             */
/*            ---------------------------------------                         */
/*           |  [S01]  |  [S02]  |  [S03]  |  [S04]  |                        */
/*           |    7    |    8    |    9    | CAL 2Kg |------> PT1.3           */
/*           |  [00]   |  [04]   |  [08]   |  [12]   |                        */
/*            ---------------------------------------                         */
/*           |  [S05]  |  [S06]  |  [S07]  |  [S08]  |                        */
/*           |    4    |    5    |    6    | CAL 0Kg |------> PT1.2           */
/*           |  [01]   |  [05]   |  [09]   |  [13]   |                        */
/*            ---------------------------------------                         */
/*           |  [S09]  |  [S10]  |  [S11]  |  [S12]  |                        */
/*           |    1    |    2    |    3    | SHOW AD |------> PT1.1           */
/*           |  [02]   |  [06]   |  [10]   |  [14]   |                        */
/*            ---------------------------------------                         */
/*           |  [S13]  |  [S14]  |  [S15]  |  [S16]  |                        */
/*           |    0    |    .    |  CLEAR  | ZERO IN |------> PT1.0           */
/*           |  [03]   |  [07]   |  [11]   |  [15]   |                        */
/*            ---------------------------------------                         */
/*                                                                            */
/*----------------------------------------------------------------------------*/
void Key_Scan_Process(void)
{
    unsigned char key_code, temp, s_bit;
    unsigned int  r_data, r_bit;

    key_code = NULL_KEY_NO;
    temp = NULL_KEY_NO;
//------Matrix scaning--------------------------
    for ( s_bit= 0x80; s_bit !=0x08; s_bit = s_bit >> 1 )
    {
        DrvGPIO_PT1_DisableOUTPUT( ~s_bit );
        DrvGPIO_PT1_EnableOUTPUT( s_bit );
        Delay( 10 );                                            //delay 18uS @4MHz.
        r_data = DrvGPIO_GetPortBits( E_PT1 );
        for ( r_bit= 0x08; r_bit > 0; r_bit = r_bit >> 1 )
        {
            temp++;
            if ( !(r_data & r_bit) )
                key_code = temp;
        }
    }

//------Key in debounce-------------------------
    Key_DB_Cnt++;
    if ( key_code != Key_Code_Buf )
    {
        Key_Code_Buf = key_code;
        Key_DB_Cnt = 0;
    }
    else
    {
        if (Key_DB_Cnt >= KEY_DB_TIME)
        {
            Key_DB_Cnt = 0;
            if ( Key_Code_Buf == NULL_KEY_NO )
            {
                UserFlag0.b_KeyOn_Flag = FALSE;
            }
            else
            {
                if (UserFlag0.b_KeyOn_Flag == FALSE)
                {
                    UserFlag0.b_KeyOn_Flag = TRUE;
                    UserFlag0.b_KInPro_Flag = TRUE;
                    Key_Code_No = Key_Code_Buf;
                }
                Sleep_Time_Cnt = SLEEP_TIME;
            }
        }
    }
    DrvGPIO_PT1_DisableOUTPUT( 0xFF );                          //close PT1[7:0] OE.
    DrvGPIO_PT1_ClrPortBits(0xF0);                              //PT1[7:4] output low.
}

/*----------------------------------------------------------------------------*/
/* Function Name: Key_In_Process                                              */
/* Description  : Key in tasks processing.                                    */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void Key_In_Process(void)
{
    unsigned char no = 0xFF;

//------Rest for newly No. key in---------------
    NoKey_Reset_Cnt++;
    if ( NoKey_Reset_Cnt > NO_KEY_RESET_TIME )
    {
        UserFlag1.b_DotKey_Flag = FALSE;
        NoKey_Reset_Cnt = 0;
        NoBuf_Idx = 2;
        NoKeyBuf[0] = 0;
        NoKeyBuf[1] = 0;
        NoKeyBuf[2] = 0;
        NoKeyBuf[3] = 0;
    }

//------Next Cal. Step display ?----------------
//step1: just delay 1500mS
//step2: show "PASS" at LCD3 about 1000mS.
//step3: release Cal. mode.
//
    if( UserFlag0.b_CalGo_Flag == TRUE )
    {
        CAL_Step_Idx += 1;

        if( CAL_Step_Idx == (1500u/32u) )
            LCD3_Display_PASS();
        else if( CAL_Step_Idx > (2500u/32u) )
            UserFlag0.b_InitGo_Flag = TRUE;

        UserFlag0.b_KInPro_Flag = FALSE;
        return;
    }

//------Key In process going--------------------
    if( UserFlag0.b_KInPro_Flag != TRUE )
        return;
    UserFlag0.b_KInPro_Flag = FALSE;

    switch( Key_Code_No )
    {
        //No. 7
        case S1_KEY_NO:
            no = 7;
            break;
        //No. 8
        case S2_KEY_NO:
            no = 8;
            break;
        //No. 9
        case S3_KEY_NO:
            no = 9;
            break;
        //CAL. 0Kg.
        case S4_KEY_NO:
        {
            if ( UserFlag1.b_CalEI_Flag == TRUE )
            {
                UserFlag0.b_CalGo_Flag = TRUE;
                CAL_Step_Idx = 0;
                LCD3_Display_CAL( 0 );
                CAL_0Kg_Buf = Check_Origin();
                ADC_Zero = ADC_Avg_RawData;
                SYS_DisableGIE();                                       //Disable Global Interrupt.
                DrvFlash_WriteWord( CAL_0KG_BUF_ADDR, CAL_0Kg_Buf );
                SYS_EnableGIE( 4, 0x06 );                               //Enable HW1 & HW2 Int. only.
            }
            break;
        }
        //No. 4
        case S5_KEY_NO:
            no = 4;
            break;
        //No. 5
        case S6_KEY_NO:
            no = 5;
            break;
        //No. 6
        case S7_KEY_NO:
            no = 6;
            break;
        //CAL. 2Kg.
        case S8_KEY_NO:
        {
            if ( UserFlag1.b_CalEI_Flag == TRUE )
            {
                UserFlag0.b_CalGo_Flag = TRUE;
                CAL_Step_Idx = 0;
                LCD3_Display_CAL( 2 );
                CAL_2Kg_Buf = Check_Origin();
                ADC_nums = ADC_Avg_RawData;
                SYS_DisableGIE();                                       //Disable Global Interrupt.
                DrvFlash_WriteWord( CAL_2KG_BUF_ADDR, CAL_2Kg_Buf );
                SYS_EnableGIE( 4, 0x06 );                               //Enable HW1 & HW2 Int. only.
            }
            break;
        }
        //No. 1
        case S9_KEY_NO:
            no = 1;
            break;
        //No. 2
        case S10_KEY_NO:
            no = 2;
            break;
        case S11_KEY_NO:
            no = 3;
            break;
        //Show ADC raw data.
        case S12_KEY_NO:
            UserFlag1.b_ShowAD_Flag = TRUE;
            break;
        //No. 0
        case S13_KEY_NO:
            no = 0;
            break;
        //"."
        case S14_KEY_NO:
        {
            UserFlag1.b_DotKey_Flag = TRUE;
            NoKey_Reset_Cnt = 0;
            break;
        }
        //Clear Setting.
        case S15_KEY_NO:
        {
            UserFlag1.b_ShowAD_Flag = FALSE;
            UserFlag1.b_DotKey_Flag = FALSE;
            NoBuf_Idx = 2;
            NoKeyBuf[0] = 0;
            NoKeyBuf[1] = 0;
            NoKeyBuf[2] = 0;
            NoKeyBuf[3] = 0;
            Unit_Price = 0;
            DecorticateBuf = 0;
            UserFlag1.b_TareEI_Flag = FALSE;
            break;
        }
        //Decorticate (Zero).
        case S16_KEY_NO:
            DecorticateBuf = Weight_Nums + DecorticateBuf;

            UserFlag1.b_TareEI_Flag = TRUE;
            break;
        default:
            break;
    }

    //No. 0~9 key in !
    if ( no < 10 )
    {
        NoKey_Reset_Cnt = 0;
        if ( UserFlag1.b_DotKey_Flag != TRUE )
        {
            NoKeyBuf[NoBuf_Idx+1] = NoKeyBuf[NoBuf_Idx];
            NoKeyBuf[NoBuf_Idx] = no;
        }
        else if ( NoBuf_Idx != 0 )
        {
            NoBuf_Idx--;
            NoKeyBuf[NoBuf_Idx] = no;
        }
        Unit_Price = NoKeyBuf[3]*1000 + NoKeyBuf[2]*100 + NoKeyBuf[1]*10 + NoKeyBuf[0];     //p
    }

    Beep_Tone_Start();
}

/*----------------------------------------------------------------------------*/
/* Function Name: DrvFlash_WriteWord                                          */
/* Description  : Write 4bytes data to Data Flash.                            */
/* Arguments    : None.                                                       */
/* Return Value : TRUE == Write pass.                                         */
/*                FALSE == Write failed.                                      */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
int DrvFlash_WriteWord(unsigned int data_addr, unsigned int data_buf)
{
    unsigned char i;

    for ( i=0; i<BURN_VERIFY_TIMES; i++ )
    {
        DrvFlash_Burn_Word( data_addr, 0x2000, data_buf );
        //Verify.
        if ( data_buf == ReadWord( data_addr ) )
            return TRUE;
    }
    return FALSE;
}

/*----------------------------------------------------------------------------*/
/* Function Name: Enter_SLEEP_Mode                                            */
/* Description  : Prepare to enter SLEEP mode.                                */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void Enter_SLEEP_Mode(void)
{
    SYS_DisableGIE();                           //Disable Global Interrupt.

    RawData_Temp = ADC_Avg_RawData;             //backup last ADC_Avg_RawData

//------Close all LCD IP------------------------
    LCD_RAM_Clear( SEG_BUF_SIZE );
    RAM2LCD( DisplayBuffer );                   //Update LCD display RAM.
    DrvLCD_DisplayMode( E_LCD_PIXELOFF );       //All LCD off.
    DrvLCD_VLCDMode( E_VLCD_DISABLE );          //close VLCD.
    while( (inw(0x41B00) & (1<<IDF) ) == 0);    //Wait LCD Idle, IDF= 20.
//------Close all ADC IP------------------------
    DrvADC_Disable();                           //disable ADC.
    DrvADC_ClkDisable();
    DrvADC_DisableInt();                        //Disable ADC interrupt.
    DrvADC_ClearIntFlag();                      //clear ADC interrupt flag.
    DrvPMU_VDDA_LDO_Ctrl( 0 );                  //Disable VDDA output.
//------Close all Timer IP----------------------
    DrvTMA_Close ();
    DrvTIMER_DisableInt( E_TMA );
    DrvTIMER_ClearIntFlag( E_TMA );
    DrvTMB_Close();
//------Initial GPIO for SLEEP wake up----------
    DrvGPIO_PT1_EnableOUTPUT( 0xF0 );           //Enable PT1[7:4] OE.
    DrvGPIO_PT1_ClearIntFlag( 0xFF );           //clear PT1[7:0] interrupt flag.
    DrvGPIO_PT1_EnableINT( 0x0F );              //PT1[3:0] interrupt enable.

    DrvPMU_LDO_LowPower(1);                     //SET low power mode
    DrvWDT_ClearWDT();                          //Clear WDT interrupt flag
}

/*----------------------------------------------------------------------------*/
/* Function Name: Release_SLEEP_Mode                                          */
/* Description  : Prepare to enter SLEEP mode.                                */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void Release_SLEEP_Mode(void)
{
    unsigned char hi_status= 0, i;
    unsigned int  r_data;

    //Debounce for PT1[3:0] wake up.
    for ( i=0; i < 250; i++ )
    {
        Delay( 100 );                              //delay 40uS @4MHz.
        r_data = DrvGPIO_GetPortBits( E_PT1 );
        if ( (r_data & 0x0F) != 0x0F )
            hi_status++;
        else
            hi_status = 0;
    }

    if ( hi_status >= 200 )
    {
        UserFlag0.b_EntSLP_Flag = FALSE;
        UserFlag0.b_InitGo_Flag = TRUE;
        UserFlag0.b_KeyOn_Flag  = TRUE;
    }
}

/*----------------------------------------------------------------------------*/
/* Function Name: Beep_Tone_Process                                           */
/* Description  : Beep tone time processing.                                  */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void Beep_Tone_Process(void)
{
    if (Beep_Tone_Cnt != 0)
    {
        Beep_Tone_Cnt--;
        if (Beep_Tone_Cnt <= 0)
            Beep_Tone_Stop();
    }
}

/*----------------------------------------------------------------------------*/
/* Function Name: Beep_Tone_Start                                             */
/* Description  : Start PWM beep tone.                                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void Beep_Tone_Start(void)
{
#if ( BEEP_TONE_ENABLE )
    Beep_Tone_Cnt = BEEP_TONE_TIME;
    DrvTMB_Open( E_TMB_MODE0, E_TMB_NORMAL, PWM_4KHZ_TIME );        //set TMB working on normal mode & TMB clock int period= PWM_4KHZ_TIME/HS_CK= 250uS.
    DrvPWM0_Open( 0, 1, 2 );                                        //Set PT2.0= PWMO0, PT2.1= PWMO1 and PWMO0 working on PWMA mdoe with normal pulse output.
    DrvPWM1_Open( 0, 0, 2 );                                        //Set PT2.0= PWMO0, PT2.1= PWMO1 and PWMO1 working on PWMA mdoe with reverse pulse output.
#endif
}

/*----------------------------------------------------------------------------*/
/* Function Name: Beep_Tone_Stop                                              */
/* Description  : Stop PWM beep tone.                                         */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void Beep_Tone_Stop(void)
{
    Beep_Tone_Cnt = 0;
    DrvTMB_Close();
    DrvPWM0_Close();
    DrvPWM1_Close();
    DrvGPIO_ClrPortBits( E_PT2, 0x03 );         //set PT2[1:0] as output low.
}

/*----------------------------------------------------------------------------*/
/* Function Name: Delay()                                                     */
/* Description  : Software delay subroutines.                                 */
/* Arguments    : unsigned int num.                                           */
/* Return Value : None.                                                       */
/* Remark       : num= 10,   delay time= 18u S @4MH HAO, 3V.                  */
/*              : num= 100,  delay time= 40u S @4MH HAO, 3V.                  */
/*              : num= 1000, delay time= 260uS @4MH HAO, 3V.                  */
/*                                                                            */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
    for( ; num >0; num-- )
        asm( "NOP" );
}

/******************************************************************************/
/* File Name    : user_GPIO.c                                                 */
/* Description  : This file implements the customer's GPIO function.          */
/* Created Date : 2016/05/28                                                  */
/* Created by   :                                                             */
/******************************************************************************/

/*----------------------------------------------------------------------------*/
/* Function Name: GPIO_Init                                                   */
/* Description  : GPIO Initialization Subroutines                             */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void GPIO_Init(void)
{
//------PT1 GPIO initial------------------------
    DrvGPIO_ClkGenerator( E_HS_CK, 3 );                         //Set IO sampling clock input source is HS_CK, and IOCLK is HS_CK/4.
    DrvGPIO_PT1_IntTriggerPorts( 0x0F, E_N_Edge );              //PT1[3:0] interrupt trigger method is negative edge.
    DrvGPIO_PT1_DisableINT( 0xFF );                             //PT1[7:0] interrupt disable.
    DrvGPIO_PT1_ClearIntFlag( 0xFF );                           //clear PT1[7:0] interrupt flag.
    DrvGPIO_PT1_EnablePullHigh( 0x0F );                         //enable PT1[3:0] pull high 75K ohm.
    DrvGPIO_PT1_EnableINPUT( 0x0F );                            //set PT1[3:0] as input port.
    DrvGPIO_PT1_DisableOUTPUT( 0xFF );                          //close PT1[7:0] OE.

//------PT2 GPIO initial------------------------
    //PWM function initial
    DrvGPIO_Open( E_PT2, 0x03, E_IO_OUTPUT );                   //set PT2[1:0] as output port.
    DrvPWM_CountCondition( PWM_4KHZ_TIME/2, PWM_4KHZ_TIME/2 );  //Set TBC1 as 50% duty, TBC2 as 50% duty.
    DrvGPIO_ClrPortBits( E_PT2, 0x03 );                         //set PT2[1:0] as output low.

    //PT2.2 Cal. sw initial
    DrvGPIO_PT2_DisableOUTPUT( 0x04 );                          //close PT2.2 OE.
    DrvGPIO_PT2_EnablePullHigh( 0x04 );                         //Enable PT2.2 pull high 75K ohm.
    DrvGPIO_PT2_EnableINPUT( 0x04 );                            //Enable PT2.2 IE.
}


/******************************************************************************/
/* File Name    : user_ADC.c                                                  */
/* Description  : This file implements the customer's ADC function.           */
/* Created Date : 2016/05/28                                                  */
/* Created by   :                                                             */
/******************************************************************************/

/*----------------------------------------------------------------------------*/
/* Function Name: ADC_Init                                                    */
/* Description  : ADC Initialization Subroutines                              */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void ADC_Init(void)
{
    //set ADC input pin
    DrvADC_PInputChannel( ADC_Input_AIO0 );     //set ADC positive input pin is AIO0.
    DrvADC_NInputChannel( ADC_Input_AIO1 );     //set ADC negative input pin is AIO1.
    DrvADC_InputSwitch( OPEN );                 //set ADC input short-switch is open
    DrvADC_RefInputShort( OPEN );               //set VREF input short-switch is open
    DrvADC_Gain( ADC_PGA_32, ADC_ADGN_4 );      //set ADC  GAIN is PGA * ADGN = 32x4 = 128.

    //set ADC DC offset
    DrvADC_DCoffset( 0 );                       //set ADC DC input offset is 0 VREF

    //set ADC input reference voltage
    DrvADC_RefVoltage ( VDDA, VSSA );           //set VREF positive input pin(REFP) is VDDA, negative input pin(REFN) is VSSA.
    DrvADC_FullRefRange( 1 );                   //set VREF gain = 1/2 VREF= 2.4V/2= 1.2V

    //set ADC comb filter
    DrvADC_OSR( 0 );                            //set OSR= ADCK/32768 = (4MHz/12)/32768 = 10 sps.
    DrvADC_CombFilter( 1 );                     //enable Comb filter.

    //set ADC clock
    DrvADC_ClkEnable( 1,1 );                    //set ADC CLOCK ADCK = HS_CK/12 = 4MHz/12 = 333KHz  & rising edge is high.

    //set VDDA voltage
    DrvPMU_VDDA_LDO_Ctrl( 3 );                  //enable adjustable LDO mode.
    DrvPMU_VDDA_Voltage( 0 );                   //set VDDA = 2.4V

    //set VREF.
    DrvPMU_AnalogGround( 1 );                   //set analog enable buffer and use internal source
    Delay( 0x1000 );

    //set ADC interrupt
    DrvADC_ClearIntFlag();                      //clear ADC interrupt flag.
    DrvADC_EnableInt();                         //enable ADC interrupt.
    DrvADC_Enable();                            //enable ADC and start ADC.
}

/*----------------------------------------------------------------------------*/
/* Function Name: ADC_RawData_Avg_Process                                     */
/* Description  : Moving average (FIR filter) for ADC raw datas.              */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       : average times = 8 times and ignore first three ADC datas.   */
/*----------------------------------------------------------------------------*/
void ADC_RawData_Avg_Process(void)
{
    if( UserFlag1.b_ADC_In_Flag == TRUE )
    {
        UserFlag1.b_ADC_In_Flag = FALSE;
        FIR_SumBuf -= ADCbuffer[FIR_Buf_Idx];
        ADCbuffer[FIR_Buf_Idx] = ADC_RawData;
        FIR_SumBuf += ADCbuffer[FIR_Buf_Idx];
        FIR_Buf_Idx += 1;
        if( FIR_Buf_Idx > (8-1) ) { FIR_Buf_Idx = 0; }
        ADC_Avg_RawData = (FIR_SumBuf >> 3);            //FIR_SumBuf/8
        asm("NOP");
    }
}


/******************************************************************************/
/* File Name    : user_TIMER.c                                                */
/* Description  : This file implements the customer's TIMER unction.          */
/* Created Date : 2016/05/28                                                  */
/* Created by   :                                                             */
/******************************************************************************/

/*----------------------------------------------------------------------------*/
/* Function Name: TIMER_Init                                                  */
/* Description  : TIMER Initialization Subroutines.                           */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void TIMER_Init(void)
{

//------Initial TIMER A for Time Base-----------
    DrvTMA_Open( 9,0 );                         //Set TimerA int period= HS_CK/32/1024= 4MHz/32768= 8mS.
    DrvTIMER_ClearIntFlag( E_TMA );             //Clear Timer A interrupt flag
    DrvTIMER_EnableInt( E_TMA );                //Timer A interrupt enable

//------Initial TIMER B for PWM-----------------
    DrvTMBC_Clk_Source( E_HS_CK, 0 );           //set TMB Clock from HS_CK/1.
    DrvTMB_Open( E_TMB_MODE0, E_TMB_NORMAL, PWM_4KHZ_TIME );        //set TMB working on normal mode & TMB clock int period= PWM_4KHZ_TIME/HS_CK= 250uS.
}

/*----------------------------------------------------------------------------*/
/* Function Name: WDT_Init                                                    */
/* Description  : WDT Initialization Subroutines.                             */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void WDT_Init(void)
{
#if ( WDT_RESET_ENABLE )
    DrvWDT_Open( E_NMI, E_PRE_SCALER_D512 );    //Enable WDT reset function and set WDT time out period = (LSRC/512)/256= 0.267Hz.
    DrvWDT_ClearWDT();                          //Clear WDT interrupt flag
#endif
}


/******************************************************************************/
/* File Name    : user_LCD.c                                                  */
/* Description  : This file implements the customer's LCD function.           */
/* Created Date : 2016/05/28                                                  */
/* Created by   :                                                             */
/******************************************************************************/

/*----------------------------------------------------------------------------*/
/* Function Name: LCD_Init                                                    */
/* Description  : LCD Initialization Subroutines                              */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       : 1. LCD working@ 4COM x 36SEG, VLCD= from charge pump 2.93V. */
/*              : 2. Charge pump's clock from HS_CK/16= 4M/16= 250KHz).       */
/*              : 3. LCD CLK(LCK) from (HS_CK/64)/9/16= 4M/9216= 434Hz.       */
/*              :    so LCD frame Freq. = 434Hz/(4x2)= 54Hz.                  */
/*----------------------------------------------------------------------------*/
void LCD_Init(void)
{
    clk_10 = 0x40404B4B;                        //Set LCK= (HS_CK/64)/9/16= (HAO/64)/9/16= (4MHz/64)/144= 434Hz & charge pump clock = HS_CK/16.
    DrvLCD_VLCDMode( E_VLCD27 );
    DrvLCD_VLCDTrim( 3 );                       //Trim VLCD as 2.93V.
    DrvLCD_LcdDuty ( E_LCD_DUTY4 );             //enable LCD@1/4 duty.

    DrvLCD_IOMode( 5, 0xFF );                   //Set SEG1/0 as LCD mode.
    DrvLCD_IOMode( 0, 0xFF );                   //Set PT6(SEG2 ~SEG9 ) as LCD mode.
    DrvLCD_IOMode( 1, 0xFF );                   //Set PT7(SEG10~SEG17) as LCD mode.
    DrvLCD_IOMode( 2, 0xFF );                   //Set PT8(SEG18~SEG25) as LCD mode.
    DrvLCD_IOMode( 3, 0xFF );                   //Set PT9(SEG26~SEG33) as LCD mode.
    DrvLCD_IOMode( 4, 0xFF );                   //Set PT10(SEG34~SEG35) as LCD mode.
    DrvLCD_LCDBuffer( ENABLE );                 //enable LCD working.
    DrvLCD_DisplayMode( E_LCD_NORMAL );         //LCD at normal mode.
    LCD_RAM_Clear( SEG_BUF_SIZE );
}

/*----------------------------------------------------------------------------*/
/* Function Name: LCD_Display_Process                                         */
/* Description  : LCD mode display going.                                     */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void LCD_Display_Process(void)
{
    LCD_Data_Display( LCD_PANEL1, Unit_Price );                     //ܳ
    Total_Price = (Unit_Price * Weight_Nums) / 1000;                //p`B
    LCD_Data_Display( LCD_PANEL2, (unsigned int)Total_Price );      //`B

    if( UserFlag0.b_CalGo_Flag != TRUE )
    {
        if( UserFlag1.b_ShowAD_Flag == FALSE )
        {
            LCD_Data_Display( LCD_PANEL3, Weight_Nums );            //`q g.
        }
        else if( UserFlag1.b_ShowAD_Flag == TRUE )
        {
            LCD_Data_Display( LCD_PANEL3, ADC_Avg_RawData );        // Avg's ADc Raw Data.
        }
    }

    RAM2LCD( DisplayBuffer );
}

/*----------------------------------------------------------------------------*/
/* Function Name: LCD_RAM_Clear                                               */
/* Description  : LCD display RAM clear.                                      */
/* Arguments    : unsigned char SegBufSize => SEG buffer size for clear.      */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void LCD_RAM_Clear(unsigned char SegBufSize)
{
    int i=0;
    for(i=0; i<SegBufSize; i++)
    {
        DisplayBuffer[i] = 0x00;
    }
}

/*----------------------------------------------------------------------------*/
/* Function Name: LCD2_Display_HYcon                                          */
/* Description  : Display "HYcon" char on LCD2 panel.                         */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void LCD2_Display_HYcon(void)
{
    DisplayBuffer[5] = Char_H;
    DisplayBuffer[6] = Char_Y;
    DisplayBuffer[7] = Char_c;
    DisplayBuffer[8] = Char_o;
    DisplayBuffer[9] = Char_n;
}

/*----------------------------------------------------------------------------*/
/* Function Name: LCD3_Display_SCALE                                          */
/* Description  : Display "SCALE" char on LCD3 panel.                         */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void LCD3_Display_SCALE(void)
{
    DisplayBuffer[11] = Char_S;
    DisplayBuffer[12] = Char_C;
    DisplayBuffer[13] = Char_A;
    DisplayBuffer[14] = Char_L;
    DisplayBuffer[15] = Char_E;
}

/*----------------------------------------------------------------------------*/
/* Function Name: LCD3_Display_ADC                                            */
/* Description  : Display ADC raw data on LCD3 panel.                         */
/* Arguments    : unsigned int adc_data=> current adc raw data for display.   */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void LCD3_Display_ADC(unsigned int adc_data)
{
    unsigned char i;

    for(i=0; i<LCD3_DIG_SIZE; i++)
    {
        DisplayBuffer[LCD3_BUF_START-i] = code_seg[adc_data%10];
        adc_data = adc_data/10;
    }

    //Over Range checking
    if ( adc_data )
    {
        for(i=0; i<LCD3_DIG_SIZE; i++)
            DisplayBuffer[LCD3_BUF_START-i] = Char_9;
    }

    DisplayBuffer[17] = 0;                      //clear sign "K".
}

/*----------------------------------------------------------------------------*/
/* Function Name: LCD3_Display_PASS                                           */
/* Description  : Display "PASS" char on LCD3 panel.                          */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void LCD3_Display_PASS(void)
{
    DisplayBuffer[11] = Char_Blank;
    DisplayBuffer[12] = Char_P;
    DisplayBuffer[13] = Char_A;
    DisplayBuffer[14] = Char_S;
    DisplayBuffer[15] = Char_S;
    DisplayBuffer[17] = 0;                      //clear sign "K".
}

/*----------------------------------------------------------------------------*/
/* Function Name: LCD3_Display_CAL                                            */
/* Description  : Display "CAL x" char on LCD3 panel.                         */
/* Arguments    : unsigned char kg_num => No. 0 or 2 for display.             */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void LCD3_Display_CAL(unsigned char kg_num)
{
    DisplayBuffer[11] = Char_C;
    DisplayBuffer[12] = Char_A;
    DisplayBuffer[13] = Char_L;
    DisplayBuffer[14] = Char_Blank;
    DisplayBuffer[15] = code_seg[ kg_num ];
    DisplayBuffer[17] |= S_K;
    DisplayBuffer[15] |= S_g;
}

/*----------------------------------------------------------------------------*/
/* Function Name: LCD_Data_Display                                            */
/* Description  : Display data on LCD panel.                                  */
/* Arguments    : unsigned char Lcd_No => No. of LCD panel.                   */
/*              : unsigned int LcdBuffer => data for display.                 */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*                                                                            */
/* LCD1 panel define:                                                         */
/*                     -----    -----    -----    -----    -----              */
/*    ////            |     |  |     |  |     |  |     |  |     |             */
/*                    |     |  |     |  |     |  |     |  |     |       >     */
/*    ----             -----    -----    -----    -----    -----        >     */
/*                    |     |  |     |  |     |  |     |  |     |       >     */
/*    TARE            |     |  |     |  |     |  |     |  |     |     g >     */
/*    ZERO             ----- o  ----- o  ----- o  ----- o  ----- o            */
/*                                                                            */
/*                     Dig4     Dig3     Dig2     Dig1     Dig0               */
/*                     Buf0     Buf1     Buf2     Buf3     Buf4               */
/*                                                                            */
/* LCD2 panel define:                                                         */
/*                     -----    -----    -----    -----    -----              */
/*                    |     |  |     |  |     |  |     |  |     |             */
/*                    |     |  |     |  |     |  |     |  |     |             */
/*                     -----    -----    -----    -----    -----              */
/*                    |     |  |     |  |     |  |     |  |     |             */
/*                    |     |  |     |  |     |  |     |  |     |     g       */
/*                     ----- o  ----- o  ----- o  ----- o  ----- o            */
/*                                                                            */
/*                     Dig4     Dig3     Dig2     Dig1     Dig0               */
/*                     Buf5     Buf6     Buf7     Buf8     Buf9               */
/*                                                                            */
/* LCD3 panel define:                                                         */
/*            -----    -----    -----    -----    -----    -----              */
/*           |     |  |     |  |     |  |     |  |     |  |     |     A       */
/*           |     |  |     |  |     |  |     |  |     |  |     |  K  X       */
/*            -----    -----    -----    -----    -----    -----      V       */
/*           |     |  |     |  |     |  |     |  |     |  |     |             */
/*           |     |  |     |  |     |  |     |  |     |  |     |     g       */
/*            ----- o  ----- o  ----- o  ----- o  ----- o  ----- o            */
/*                                                                            */
/*            Dig5     Dig4     Dig3     Dig2     Dig1     Dig0               */
/*            Buf10    Buf11    Buf12    Buf13    Buf14    Buf15              */
/*                                                                            */
/*----------------------------------------------------------------------------*/
void LCD_Data_Display(unsigned char lcd_no, unsigned int LcdBuffer)
{
    unsigned char Buf_Start, Dig_Size, Point_BIdx, i;

    DisplayBuffer[17] = 0;                  //preclear LCD3's "K" & "TARE" sign bit.
    switch( lcd_no )
    {
        //LCD1
        case LCD_PANEL1:
            Buf_Start = LCD1_BUF_START;
            Dig_Size = LCD1_DIG_SIZE;
            Point_BIdx = LCD1_POINT_BUF;
            break;
        //LCD2
        case LCD_PANEL2:
            Buf_Start = LCD2_BUF_START;
            Dig_Size = LCD2_DIG_SIZE;
            Point_BIdx = LCD2_POINT_BUF;
            break;
        //LCD3
        case LCD_PANEL3:
            Buf_Start = LCD3_BUF_START;
            Dig_Size = LCD3_DIG_SIZE;
            Point_BIdx = LCD3_POINT_BUF;
            break;
        default:
            break;
    }

    for(i=0; i<Dig_Size; i++)
    {
        DisplayBuffer[Buf_Start-i] = code_seg[LcdBuffer%10];
        LcdBuffer = LcdBuffer/10;
    }

    //Over Range checking
    if ( LcdBuffer )
    {
        for(i=0; i<Dig_Size; i++)
            DisplayBuffer[Buf_Start-i] = Char_9;
    }
    //display "g".
    if ( (lcd_no == LCD_PANEL3 ) && ( UserFlag1.b_CalEI_Flag == FALSE ) && ( UserFlag1.b_ShowAD_Flag == FALSE ) )
    {
        DisplayBuffer[17] |= S_K;
        DisplayBuffer[15] |= S_g;
        DisplayBuffer[Point_BIdx] |= seg_h;             //point display.
        if ( UserFlag1.b_TareEI_Flag == TRUE )
            DisplayBuffer[17] |= S_Tare;                //Sign "TARE".
    }
    if ( lcd_no != LCD_PANEL3 )
        DisplayBuffer[Point_BIdx] |= seg_h;             //point display.
}

/*----------------------------------------------------------------------------*/
/* Function Name: RAM2LCD                                                     */
/* Description  : RAM buffer data transfer to LCD RAM                         */
/* Arguments    : unsigned char *Buffer_Adr => buffer pointer address.        */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void RAM2LCD( unsigned char *Buffer_Adr )
{
    unsigned char buf_idx;

    for(buf_idx=0; buf_idx<SEG_BUF_SIZE; buf_idx++)
    {
        DrvLCD_WriteData ( buf_idx, *Buffer_Adr );
        Buffer_Adr++;
    }
}

/*----------------------------------------------------------------------------*/
/* End Of File                                                                */
/*----------------------------------------------------------------------------*/
